const tar = require('tar');
const fetch = require('node-fetch');
const path = require('path');
const getWritableDirectory = require('@now/build-utils/fs/get-writable-directory.js');
const { cli, commandInDir } = require('./cli');
const installOlderRuby = require('./apt-install-ruby')


const rubyURL = 'https://cache.ruby-lang.org/pub/ruby/2.6/ruby-2.6.2.tar.gz'

async function getRuby(dir) {
	console.log('downloading ruby.........')
  const res = await fetch(rubyURL)

  if (!res.ok) {
    throw new Error(`Failed to download: ${rubyURL}`);
  }

  return new Promise((resolve, reject) => {
    console.log("In Promise")
    res.body
      .on('error', reject)
      .pipe(tar.extract({ gzip: true, cwd: dir}))
      .on('finish', () => resolve(path.join(dir, 'ruby-2.6.2')))
  })
}

async function compileRuby(dir) {
  const executor = commandInDir(dir)
  console.log('about to configure')
  await executor('chmod', '+x', 'configure')
  // $ ./configure
  await executor('./configure')
  // $ make
  await executor('make')
  // $ make check == "check succeeded" if successful
  // await executor('make', 'check')
  // $ make update-gems
  await executor('make', 'update-gems')
  // $ make extract-gems
  await executor('make', 'extract-gems')
  // $ sudo make install
  await executor('make', 'install')
  // rm -rf ruby-2.6.2*
  const upOneLevel = path.resolve(dir, "..")
  await commandInDir(upOneLevel)('rm', '-rf', 'ruby-2.6.2*')

  return cli('ruby', '-v')
}


module.exports = async () => {
  const dir = await getWritableDirectory()
  try {
    const olderVersion = await installOlderRuby()
    console.log(`Older Version installed: ${olderVersion}`)
    const rubyPath = await getRuby(dir)
    console.log('After download')
    const installedRubyVersion = await compileRuby(rubyPath)
    console.log('After compile')
    console.log(`successfully installed Ruby with version ${installedRubyVersion}`)
  }
  catch (err){
    console.log(err)
    throw err
  }
}